@@ -163,13 +163,13 @@ enum
     NET_WM_MOVERESIZE_SIZE_LEFT,
     NET_WM_MOVERESIZE_MOVE,
     NET_WM_MOVERESIZE_SIZE_KEYBOARD,
-    NET_WM_MOVERESIZE_MOVE_KEYBOARD
+    NET_WM_MOVERESIZE_MOVE_KEYBOARD,
+    NET_WM_MOVERESIZE_CANCEL
 };
 
 enum
 {
     COMPOSITING_MANAGER = 0,
-    GNOME_PANEL_DESKTOP_AREA,
     KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR,
     KWM_WIN_ICON,
     MOTIF_WM_HINTS,
@@ -250,15 +250,6 @@ enum
     PIXMAP,
     SM_CLIENT_ID,
     UTF8_STRING,
-    WIN_CLIENT_LIST,
-    WIN_DESKTOP_BUTTON_PROXY,
-    WIN_HINTS,
-    WIN_LAYER,
-    WIN_PROTOCOLS,
-    WIN_STATE,
-    WIN_SUPPORTING_WM_CHECK,
-    WIN_WORKSPACE,
-    WIN_WORKSPACE_COUNT,
     WM_CHANGE_STATE,
     WM_CLIENT_LEADER,
     WM_CLIENT_MACHINE,
@@ -363,6 +354,8 @@ DisplayInfo             *myDisplayClose                         (DisplayInfo *);
 gboolean                 myDisplayHaveShape                     (DisplayInfo *);
 gboolean                 myDisplayHaveShapeInput                (DisplayInfo *);
 gboolean                 myDisplayHaveRender                    (DisplayInfo *);
+void                     myDisplayCreateCursor                  (DisplayInfo *);
+void                     myDisplayFreeCursor                    (DisplayInfo *);
 Cursor                   myDisplayGetCursorBusy                 (DisplayInfo *);
 Cursor                   myDisplayGetCursorMove                 (DisplayInfo *);
 Cursor                   myDisplayGetCursorRoot                 (DisplayInfo *);
