@@ -63,6 +63,7 @@
 #include "events.h"
 #include "event_filter.h"
 #include "xsync.h"
+#include "display.h"
 
 #ifndef CHECK_BUTTON_TIME
 #define CHECK_BUTTON_TIME 0
@@ -74,7 +75,7 @@
                                  (win == MYWINDOW_XWINDOW(c->buttons[SHADE_BUTTON])) || \
                                  (win == MYWINDOW_XWINDOW(c->buttons[STICK_BUTTON])))
 
-#define DOUBLE_CLICK_GRAB          (ButtonMotionMask | \
+#define DOUBLE_CLICK_GRAB       (ButtonMotionMask | \
                                  PointerMotionMask | \
                                  ButtonPressMask | \
                                  ButtonReleaseMask)
@@ -138,7 +139,6 @@ static gboolean
 typeOfClick_end (gpointer data)
 {
     XfwmButtonClickData *passdata;
-    TRACE ("typeOfClick_end(): Exit typeOfClick() event loop");
 
     passdata = (XfwmButtonClickData *) data;
     if (passdata->timeout)
@@ -149,7 +149,7 @@ typeOfClick_end (gpointer data)
 
     gtk_main_quit ();
 
-    return (TRUE);
+    return (FALSE);
 }
 
 static eventFilterStatus
@@ -162,13 +162,19 @@ typeOfClick_event_filter (XEvent * xevent, gpointer data)
 
     keep_going = TRUE;
     passdata = (XfwmButtonClickData *) data;
-    status = EVENT_FILTER_STOP;
+    status = EVENT_FILTER_CONTINUE;
 
     /* Update the display time */
     timestamp = myDisplayUpdateCurrentTime (passdata->display_info, xevent);
-    if (timestamp)
-        passdata->tcurrent = timestamp;
 
+    if (timestamp)
+    {
+        passdata->tcurrent = timestamp;
+        if (((gint) passdata->tcurrent - (gint) passdata->t0) > passdata->double_click_time)
+        {
+            keep_going = FALSE;
+        }
+    }
     if ((xevent->type == ButtonRelease) || (xevent->type == ButtonPress))
     {
         if (xevent->xbutton.button == passdata->button)
@@ -181,11 +187,19 @@ typeOfClick_event_filter (XEvent * xevent, gpointer data)
         {
             keep_going = FALSE;
         }
+        status = EVENT_FILTER_STOP;
     }
     else if (xevent->type == MotionNotify)
     {
         passdata->xcurrent = xevent->xmotion.x_root;
         passdata->ycurrent = xevent->xmotion.y_root;
+
+        if ((ABS (passdata->x0 - passdata->xcurrent) > passdata->double_click_distance) ||
+            (ABS (passdata->y0 - passdata->ycurrent) > passdata->double_click_distance))
+        {
+            keep_going = FALSE;
+        }
+        status = EVENT_FILTER_STOP;
     }
     else if ((xevent->type == DestroyNotify) || (xevent->type == UnmapNotify))
     {
@@ -195,17 +209,9 @@ typeOfClick_event_filter (XEvent * xevent, gpointer data)
             passdata->clicks = (guint) XFWM_BUTTON_UNDEFINED;
             keep_going = FALSE;
         }
-        status = EVENT_FILTER_CONTINUE;
-    }
-    else
-    {
-        status = EVENT_FILTER_CONTINUE;
     }
 
-    if ((ABS (passdata->x0 - passdata->xcurrent) > passdata->double_click_distance) ||
-        (ABS (passdata->y0 - passdata->ycurrent) > passdata->double_click_distance) ||
-        (((gint) passdata->tcurrent - (gint) passdata->t0) > passdata->double_click_time) ||
-        (!keep_going))
+    if (!keep_going)
     {
         TRACE ("click type=%u", passdata->clicks);
         TRACE ("time=%ims (timeout=%ims)", (gint) passdata->tcurrent - (gint) passdata->t0, passdata->double_click_time);
@@ -229,7 +235,6 @@ typeOfClick (ScreenInfo *screen_info, Window w, XEvent * ev, gboolean allow_doub
     g_return_val_if_fail (w != None, XFWM_BUTTON_UNDEFINED);
 
     display_info = screen_info->display_info;
-    XFlush (display_info->dpy);
     g = myScreenGrabPointer (screen_info, DOUBLE_CLICK_GRAB, None, ev->xbutton.time);
 
     if (!g)
@@ -255,8 +260,7 @@ typeOfClick (ScreenInfo *screen_info, Window w, XEvent * ev, gboolean allow_doub
     passdata.double_click_distance = display_info->double_click_distance;
     TRACE ("Double click time= %i, distance=%i\n", display_info->double_click_time,
                                                    display_info->double_click_distance);
-    /* Use G_PRIORITY_DEFAULT_IDLE to make sure the timeout occurs after possible events */
-    passdata.timeout =  g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE,
+    passdata.timeout = g_timeout_add_full (G_PRIORITY_HIGH,
                                            display_info->double_click_time,
                                            (GSourceFunc) typeOfClick_end,
                                            (gpointer) &passdata, NULL);
@@ -268,7 +272,7 @@ typeOfClick (ScreenInfo *screen_info, Window w, XEvent * ev, gboolean allow_doub
     TRACE ("leaving typeOfClick loop");
 
     myScreenUngrabPointer (screen_info, myDisplayGetCurrentTime (display_info));
-    XFlush (display_info->dpy);
+
     return (XfwmButtonClickType) passdata.clicks;
 }
 
@@ -298,7 +302,7 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
     Client *c;
     int key;
 
-    TRACE ("entering handleKeyEvent");
+    TRACE ("entering handleKeyPress");
 
     ev_screen_info = myDisplayGetScreenFromRoot (display_info, ev->root);
     if (!ev_screen_info)
@@ -336,13 +340,13 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
                 }
                 break;
             case KEY_MAXIMIZE_WINDOW:
-                clientToggleMaximized (c, WIN_STATE_MAXIMIZED, TRUE);
+                clientToggleMaximized (c, CLIENT_FLAG_MAXIMIZED, TRUE);
                 break;
             case KEY_MAXIMIZE_VERT:
-                clientToggleMaximized (c, WIN_STATE_MAXIMIZED_VERT, TRUE);
+                clientToggleMaximized (c, CLIENT_FLAG_MAXIMIZED_VERT, TRUE);
                 break;
             case KEY_MAXIMIZE_HORIZ:
-                clientToggleMaximized (c, WIN_STATE_MAXIMIZED_HORIZ, TRUE);
+                clientToggleMaximized (c, CLIENT_FLAG_MAXIMIZED_HORIZ, TRUE);
                 break;
             case KEY_SHADE_WINDOW:
                 clientToggleShaded (c);
@@ -360,6 +364,16 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
             case KEY_LOWER_WINDOW:
                 clientLower (c, None);
                 break;
+            case KEY_RAISELOWER_WINDOW:
+                if (clientIsTopMost (c))
+                {
+                    clientLower (c, None);
+                }
+                else
+                {
+                    clientRaise (c, None);
+                }
+                break;
             case KEY_TOGGLE_ABOVE:
                 clientToggleLayerAbove (c);
                 break;
@@ -414,7 +428,27 @@ handleKeyPress (DisplayInfo *display_info, XKeyEvent * ev)
                 clientFill (c, CLIENT_FILL_VERT);
                 break;
             case KEY_FILL_HORIZ:
-                    clientFill (c, CLIENT_FILL_HORIZ);
+                clientFill (c, CLIENT_FILL_HORIZ);
+                break;
+            case KEY_TILE_DOWN:
+                clientTile (c, frameX (c) + frameWidth (c) / 2,
+                               frameY (c) + frameHeight (c) / 2,
+                               TILE_DOWN, TRUE);
+                break;
+            case KEY_TILE_LEFT:
+                clientTile (c, frameX (c) + frameWidth (c) / 2,
+                               frameY (c) + frameHeight (c) / 2,
+                               TILE_LEFT, TRUE);
+                break;
+            case KEY_TILE_RIGHT:
+                clientTile (c, frameX (c) + frameWidth (c) / 2,
+                               frameY (c) + frameHeight (c) / 2,
+                               TILE_RIGHT, TRUE);
+                break;
+            case KEY_TILE_UP:
+                clientTile (c, frameX (c) + frameWidth (c) / 2,
+                               frameY (c) + frameHeight (c) / 2,
+                               TILE_UP, TRUE);
                 break;
             default:
                 break;
@@ -673,7 +707,6 @@ static void
 button1Action (Client * c, XButtonEvent * ev)
 {
     ScreenInfo *screen_info;
-    XEvent copy_event;
     XfwmButtonClickType tclick;
 
     g_return_if_fail (c != NULL);
@@ -687,11 +720,8 @@ button1Action (Client * c, XButtonEvent * ev)
     }
     clientRaise (c, None);
 
-    memcpy(&copy_event, ev, sizeof(XEvent));
-    tclick = typeOfClick (screen_info, c->window, &copy_event, TRUE);
-
-    if ((tclick == XFWM_BUTTON_DRAG)
-        || (tclick == XFWM_BUTTON_CLICK_AND_DRAG))
+    tclick = typeOfClick (screen_info, c->window, (XEvent *) ev, TRUE);
+    if ((tclick == XFWM_BUTTON_DRAG) || (tclick == XFWM_BUTTON_CLICK_AND_DRAG))
     {
         clientMove (c, (XEvent *) ev);
     }
@@ -700,7 +730,7 @@ button1Action (Client * c, XButtonEvent * ev)
         switch (screen_info->params->double_click_action)
         {
             case DOUBLE_CLICK_ACTION_MAXIMIZE:
-                clientToggleMaximized (c, WIN_STATE_MAXIMIZED, TRUE);
+                clientToggleMaximized (c, CLIENT_FLAG_MAXIMIZED, TRUE);
                 break;
             case DOUBLE_CLICK_ACTION_SHADE:
                 clientToggleShaded (c);
@@ -741,17 +771,9 @@ titleButton (Client * c, guint state, XButtonEvent * ev)
     }
     else if (ev->button == Button3)
     {
-        /*
-           We need to copy the event to keep the original event untouched
-           for gtk to handle it (in case we open up the menu)
-         */
-
-        XEvent copy_event;
         XfwmButtonClickType tclick;
 
-        memcpy(&copy_event, ev, sizeof(XEvent));
-        tclick = typeOfClick (screen_info, c->window, &copy_event, FALSE);
-
+        tclick = typeOfClick (screen_info, c->window, (XEvent *) ev, FALSE);
         if (tclick == XFWM_BUTTON_DRAG)
         {
             clientMove (c, (XEvent *) ev);
@@ -881,16 +903,18 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
             part = edgeGetPart (c, ev);
             edgeButton (c, part, ev);
         }
-#if 0   /* Binding the alt+scroll wheel to switch app/window is not handy, disabling for now */
-        else if ((ev->button == Button4) && (state) && (state == screen_info->params->easy_click))
+#ifdef HAVE_COMPOSITOR
+        else if ((ev->button == Button4) && (state) && (state == screen_info->params->easy_click)
+                  && (screen_info->compositor_active))
         {
-            clientSwitchWindow ();
+            compositorZoomIn(screen_info, ev);
         }
-        else if ((ev->button == Button5) && (state) && (state == screen_info->params->easy_click))
+        else if ((ev->button == Button5) && (state) && (state == screen_info->params->easy_click)
+                  && (screen_info->compositor_active))
         {
-            clientSwitchApp ();
+            compositorZoomOut(screen_info, ev);
         }
-#endif
+#endif /* HAVE_COMPOSITOR */
         else if (WIN_IS_BUTTON (win))
         {
             if (ev->button <= Button3)
@@ -915,16 +939,9 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
         {
             if (ev->button == Button1)
             {
-                /*
-                   We need to copy the event to keep the original event untouched
-                   for gtk to handle it (in case we open up the menu)
-                 */
-
-                XEvent copy_event;
                 XfwmButtonClickType tclick;
 
-                memcpy(&copy_event, ev, sizeof(XEvent));
-                tclick = typeOfClick (screen_info, c->window, &copy_event, TRUE);
+                tclick = typeOfClick (screen_info, c->window, (XEvent *) ev, TRUE);
 
                 if (tclick == XFWM_BUTTON_DOUBLE_CLICK)
                 {
@@ -995,7 +1012,6 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
         else if (ev->window == c->window)
         {
             replay = TRUE;
-            clientPassGrabMouseButton (c);
             if (((screen_info->params->raise_with_any_button) && (c->type & WINDOW_REGULAR_FOCUSABLE)) || (ev->button == Button1))
             {
                 if (!(c->type & WINDOW_TYPE_DONT_FOCUS))
@@ -1368,7 +1384,7 @@ handleEnterNotify (DisplayInfo *display_info, XCrossingEvent * ev)
                 if(screen_info->params->focus_delay)
                 {
                     clientClearDelayedFocus ();
-                    clientAddDelayedFocus (c);
+                    clientAddDelayedFocus (c, ev->time);
                 }
                 else
                 {
@@ -1519,7 +1535,6 @@ handleEnterNotify (DisplayInfo *display_info, XCrossingEvent * ev)
         if (warp_pointer)
         {
             XWarpPointer (display_info->dpy, None, None, 0, 0, 0, 0, rx, ry);
-            XFlush (display_info->dpy);
         }
     }
 
@@ -1728,7 +1743,6 @@ handleFocusOut (DisplayInfo *display_info, XFocusChangeEvent * ev)
         if ((c) && (c == clientGetFocus ()))
         {
             TRACE ("focus lost from \"%s\" (0x%lx)", c->name, c->window);
-            clientPassGrabMouseButton (NULL);
             clientUpdateFocus (c->screen_info, NULL, NO_FOCUS_FLAG);
             clientClearDelayedRaise ();
         }
@@ -1811,16 +1825,14 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
             Window w;
 
             TRACE ("client \"%s\" (0x%lx) has received a WM_TRANSIENT_FOR notify", c->name, c->window);
+            c->transient_for = None;
             getTransientFor (display_info, c->screen_info->xroot, c->window, &w);
             if (clientCheckTransientWindow (c, w))
             {
                 c->transient_for = w;
             }
-        }
-        else if (ev->atom == display_info->atoms[WIN_HINTS])
-        {
-            TRACE ("client \"%s\" (0x%lx) has received a WIN_HINTS notify", c->name, c->window);
-            getHint (display_info, c->window, WIN_HINTS, (long *) &c->win_hints);
+            /* Recompute window type as it may have changed */
+            clientWindowType (c);
         }
         else if (ev->atom == display_info->atoms[NET_WM_WINDOW_TYPE])
         {
@@ -1932,12 +1944,6 @@ handlePropertyNotify (DisplayInfo *display_info, XPropertyEvent * ev)
             workspaceSetNames (screen_info, names, items);
         }
     }
-    else if (ev->atom == display_info->atoms[GNOME_PANEL_DESKTOP_AREA])
-    {
-        TRACE ("root has received a GNOME_PANEL_DESKTOP_AREA notify");
-        getGnomeDesktopMargins (display_info, screen_info->xroot, screen_info->gnome_margins);
-        workspaceUpdateArea (screen_info);
-    }
     else if (ev->atom == display_info->atoms[NET_DESKTOP_LAYOUT])
     {
         TRACE ("root has received a NET_DESKTOP_LAYOUT notify");
@@ -1956,7 +1962,13 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
     Client *c;
 
     TRACE ("entering handleClientMessage");
+    TRACE ("handleClientMessage from window (0x%lx)", ev->window);
 
+    if (ev->window == None)
+    {
+        /* Some do not set the window member, not much we can do without */
+        return EVENT_FILTER_PASS;
+    }
     status = EVENT_FILTER_PASS;
     c = myDisplayGetClientFromWindow (display_info, ev->window, SEARCH_WINDOW);
     if (c)
@@ -1972,56 +1984,10 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
                 clientWithdraw (c, c->win_workspace, TRUE);
             }
         }
-        else if ((ev->message_type == display_info->atoms[WIN_STATE]) && (ev->format == 32))
-        {
-            TRACE ("client \"%s\" (0x%lx) has received a WIN_STATE event", c->name, c->window);
-            clientUpdateWinState (c, ev);
-        }
-        else if ((ev->message_type == display_info->atoms[WIN_LAYER]) && (ev->format == 32))
-        {
-            TRACE ("client \"%s\" (0x%lx) has received a WIN_LAYER event", c->name, c->window);
-            /*
-             * Some apps that I wouldn't name try to manipulate the win layer by themselves
-             * and cause havoc when doing so on transient dialogs, so we need to be extra careful
-             * here before allowing apps to change the layer.
-             * Actually, I beleive twe should get rid of support of this old protocol...
-             */
-            if (!clientIsTransientOrModal(c) && ((unsigned long) ev->data.l[0] != c->win_layer))
-            {
-                clientSetLayer (c, ev->data.l[0]);
-            }
-        }
-        else if ((ev->message_type == display_info->atoms[WIN_WORKSPACE]) && (ev->format == 32))
-        {
-            TRACE ("client \"%s\" (0x%lx) has received a WIN_WORKSPACE event", c->name, c->window);
-            if (ev->data.l[0] != (long) c->win_workspace)
-            {
-                clientSetWorkspace (c, (guint) ev->data.l[0], TRUE);
-            }
-        }
         else if ((ev->message_type == display_info->atoms[NET_WM_DESKTOP]) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a NET_WM_DESKTOP event", c->name, c->window);
-            if (ev->data.l[0] == ALL_WORKSPACES)
-            {
-                if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK) && !FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
-                {
-                    clientStick (c, TRUE);
-                    frameQueueDraw (c, FALSE);
-                }
-            }
-            else
-            {
-                if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK) && FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
-                {
-                    clientUnstick (c, TRUE);
-                    frameQueueDraw (c, FALSE);
-                }
-                if (ev->data.l[0] != (long) c->win_workspace)
-                {
-                    clientSetWorkspace (c, (guint) ev->data.l[0], TRUE);
-                }
-            }
+            clientUpdateNetWmDesktop (c, ev);
         }
         else if ((ev->message_type == display_info->atoms[NET_CLOSE_WINDOW]) && (ev->format == 32))
         {
@@ -2070,8 +2036,7 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
         }
 
         status = EVENT_FILTER_REMOVE;
-        if (((ev->message_type == display_info->atoms[WIN_WORKSPACE]) ||
-             (ev->message_type == display_info->atoms[NET_CURRENT_DESKTOP])) && (ev->format == 32))
+        if ((ev->message_type == display_info->atoms[NET_CURRENT_DESKTOP]) && (ev->format == 32))
         {
             TRACE ("root has received a win_workspace or a NET_CURRENT_DESKTOP event %li", ev->data.l[0]);
             if ((ev->data.l[0] >= 0) && (ev->data.l[0] < (long) screen_info->workspace_count) &&
@@ -2081,8 +2046,7 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
                                  myDisplayGetTime (display_info, (guint32) ev->data.l[1]));
             }
         }
-        else if (((ev->message_type == display_info->atoms[WIN_WORKSPACE_COUNT]) ||
-                  (ev->message_type == display_info->atoms[NET_NUMBER_OF_DESKTOPS])) && (ev->format == 32))
+        else if ((ev->message_type == display_info->atoms[NET_NUMBER_OF_DESKTOPS]) && (ev->format == 32))
         {
             TRACE ("root has received a win_workspace_count event");
             if (ev->data.l[0] != (long) screen_info->workspace_count)
@@ -2419,7 +2383,7 @@ menu_callback (Menu * menu, MenuOp op, Window xid, gpointer menu_data, gpointer
             case MENU_OP_UNMAXIMIZE:
                 if (CLIENT_CAN_MAXIMIZE_WINDOW (c))
                 {
-                    clientToggleMaximized (c, WIN_STATE_MAXIMIZED, TRUE);
+                    clientToggleMaximized (c, CLIENT_FLAG_MAXIMIZED, TRUE);
                 }
                 break;
             case MENU_OP_MINIMIZE:
@@ -2756,6 +2720,26 @@ double_click_distance_cb (GObject * obj, GdkEvent * ev, gpointer data)
     return (TRUE);
 }
 
+static void
+cursor_theme_cb (GObject * obj, GParamSpec * pspec, gpointer data)
+{
+    DisplayInfo * display_info;
+    GSList *list;
+
+    display_info = (DisplayInfo *) data;
+    g_return_if_fail (display_info);
+
+    myDisplayFreeCursor (display_info);
+    myDisplayCreateCursor (display_info);
+
+    for (list = display_info->screens; list; list = g_slist_next (list))
+    {
+        ScreenInfo *screen_info = (ScreenInfo *) list->data;
+        clientUpdateAllCursor (screen_info);
+        XDefineCursor (display_info->dpy, screen_info->xroot, display_info->root_cursor);
+   }
+}
+
 static gboolean
 client_event_cb (GtkWidget * widget, GdkEventClient * ev, gpointer data)
 {
@@ -2777,13 +2761,19 @@ client_event_cb (GtkWidget * widget, GdkEventClient * ev, gpointer data)
 static gboolean
 refresh_font_cb (GObject * obj, GdkEvent * ev, gpointer data)
 {
-    ScreenInfo *screen_info;
+    DisplayInfo * display_info;
+    GSList *list;
 
-    screen_info = (ScreenInfo *) data;
-    g_return_val_if_fail (screen_info, TRUE);
+    display_info = (DisplayInfo *) data;
+    g_return_val_if_fail (display_info, TRUE);
+
+    for (list = display_info->screens; list; list = g_slist_next (list))
+    {
+        ScreenInfo *screen_info = (ScreenInfo *) list->data;
+        myScreenUpdateFontHeight (screen_info);
+        clientUpdateAllFrames (screen_info, UPDATE_FRAME);
+    }
 
-    myScreenUpdateFontHeight (screen_info);
-    clientUpdateAllFrames (screen_info, UPDATE_FRAME);
     return (TRUE);
 }
 
@@ -2887,47 +2877,54 @@ monitors_changed_cb(GdkScreen *gscreen, gpointer data)
 }
 
 void
-initGtkCallbacks (ScreenInfo *screen_info)
+initPerScreenCallbacks (ScreenInfo *screen_info)
+{
+    g_return_if_fail (screen_info);
+
+    screen_info->button_handler_id =
+        g_signal_connect (G_OBJECT (myScreenGetGtkWidget (screen_info)),
+                          "button_press_event", GTK_SIGNAL_FUNC (show_popup_cb), (gpointer) NULL);
+    g_signal_connect (G_OBJECT (myScreenGetGtkWidget (screen_info)), "client_event",
+                      GTK_SIGNAL_FUNC (client_event_cb), NULL);
+    g_object_connect (G_OBJECT(screen_info->gscr),
+                      "signal::size-changed",
+                      G_CALLBACK(size_changed_cb), (gpointer) (screen_info),
+                      "signal::monitors-changed",
+                      G_CALLBACK(monitors_changed_cb), (gpointer) (screen_info),
+                      NULL);
+}
+
+void
+initPerDisplayCallbacks (DisplayInfo *display_info)
 {
     GtkSettings *settings;
 
-    screen_info->button_handler_id =
-        g_signal_connect (GTK_OBJECT (myScreenGetGtkWidget (screen_info)),
-                          "button_press_event", GTK_SIGNAL_FUNC (show_popup_cb), (gpointer) NULL);
-    g_signal_connect (GTK_OBJECT (myScreenGetGtkWidget (screen_info)), "client_event",
-                      GTK_SIGNAL_FUNC (client_event_cb), (gpointer) (screen_info->display_info));
-    g_signal_connect (G_OBJECT(screen_info->gscr), "size-changed",
-                      G_CALLBACK(size_changed_cb),
-                      (gpointer) (screen_info));
-    if (gtk_major_version > 2 || (gtk_major_version == 2 && gtk_minor_version >= 13))
-    {
-        TRACE ("connect \"monitors-changed\" cb");
-        g_signal_connect (G_OBJECT(screen_info->gscr), "monitors-changed",
-                          G_CALLBACK(monitors_changed_cb),
-                          (gpointer) (screen_info));
-    }
+    g_return_if_fail (display_info);
 
     settings = gtk_settings_get_default ();
-    if (settings)
-    {
-        g_signal_connect (settings, "notify::gtk-theme-name",
-            G_CALLBACK (set_reload), (gpointer) (screen_info->display_info));
-        g_signal_connect (settings, "notify::gtk-font-name",
-            G_CALLBACK (set_reload), (gpointer) (screen_info->display_info));
-        g_signal_connect (settings, "notify::gtk-double-click-time",
-            G_CALLBACK (double_click_time_cb), (gpointer) (screen_info->display_info));
-        g_signal_connect (settings, "notify::gtk-double-click-distance",
-            G_CALLBACK (double_click_distance_cb), (gpointer) (screen_info->display_info));
-        g_signal_connect_after (settings, "notify::gtk-xft-antialias",
-            G_CALLBACK (refresh_font_cb), (gpointer) (screen_info));
-        g_signal_connect_after (settings, "notify::gtk-xft-dpi",
-            G_CALLBACK (refresh_font_cb), (gpointer) (screen_info));
-        g_signal_connect_after (settings, "notify::gtk-xft-hinting",
-            G_CALLBACK (refresh_font_cb), (gpointer) (screen_info));
-        g_signal_connect_after (settings, "notify::gtk-xft-hintstyle",
-            G_CALLBACK (refresh_font_cb), (gpointer) (screen_info));
-        g_signal_connect_after (settings, "notify::gtk-xft-rgba",
-            G_CALLBACK (refresh_font_cb), (gpointer) (screen_info));
-    }
+    g_object_connect (settings,
+                      "signal::notify::gtk-theme-name",
+                      G_CALLBACK (set_reload),  (gpointer) (display_info),
+                      "signal::notify::gtk-font-name",
+                      G_CALLBACK (set_reload), (gpointer) (display_info),
+                      "signal::notify::gtk-double-click-time",
+                      G_CALLBACK (double_click_time_cb), (gpointer) (display_info),
+                      "signal::notify::gtk-double-click-distance",
+                      G_CALLBACK (double_click_distance_cb), (gpointer) (display_info),
+                      "signal::notify::gtk-cursor-theme-name",
+                      G_CALLBACK (cursor_theme_cb), (gpointer) (display_info),
+                      "signal::notify::gtk-cursor-theme-size",
+                      G_CALLBACK (cursor_theme_cb), (gpointer) (display_info),
+                      "signal::notify::gtk-xft-antialias",
+                      G_CALLBACK (refresh_font_cb), (gpointer) (display_info),
+                      "signal::notify::gtk-xft-dpi",
+                      G_CALLBACK (refresh_font_cb), (gpointer) (display_info),
+                      "signal::notify::gtk-xft-hinting",
+                      G_CALLBACK (refresh_font_cb), (gpointer) (display_info),
+                      "signal::notify::gtk-xft-hintstyle",
+                      G_CALLBACK (refresh_font_cb), (gpointer) (display_info),
+                      "signal::notify::gtk-xft-rgba",
+                      G_CALLBACK (refresh_font_cb), (gpointer) (display_info),
+                      NULL);
 }
 
