@@ -73,7 +73,7 @@
     KeyReleaseMask|\
     FocusChangeMask|\
     PropertyChangeMask|\
-    ColormapNotify
+    ColormapChangeMask
 
 #ifdef HAVE_COMPOSITOR
 #define MAIN_EVENT_MASK BASE_EVENT_MASK|ExposureMask
