@@ -55,7 +55,7 @@
     ButtonReleaseMask | \
     LeaveWindowMask
 
-#define TILE_DISTANCE 15
+#define TILE_DISTANCE 2
 
 typedef struct _MoveResizeData MoveResizeData;
 struct _MoveResizeData
@@ -71,6 +71,7 @@ struct _MoveResizeData
     gint cancel_w, cancel_h;
     guint cancel_workspace;
     gint mx, my;
+    gint px, py; /* pointer relative position */
     gint ox, oy;
     gint ow, oh;
     gint oldw, oldh;
@@ -179,40 +180,40 @@ clientSetHandle(MoveResizeData *passdata, int handle)
     switch (handle)
     {
         case CORNER_BOTTOM_LEFT:
-            px = frameX(c) + frameLeft(c) / 2;
-            py = frameY(c) + frameHeight(c) - frameBottom(c) / 2;
+            px = frameX (c) + frameLeft(c) / 2;
+            py = frameY (c) + frameHeight (c) - frameBottom(c) / 2;
             break;
         case CORNER_BOTTOM_RIGHT:
-            px = frameX(c) + frameWidth(c) - frameRight(c) / 2;
-            py = frameY(c) + frameHeight(c) - frameBottom(c) / 2;
+            px = frameX (c) + frameWidth (c) - frameRight(c) / 2;
+            py = frameY (c) + frameHeight (c) - frameBottom(c) / 2;
             break;
         case CORNER_TOP_LEFT:
-            px = frameX(c) + frameLeft(c) / 2;
-            py = frameY(c);
+            px = frameX (c) + frameLeft(c) / 2;
+            py = frameY (c);
             break;
         case CORNER_TOP_RIGHT:
-            px = frameX(c) + frameWidth(c) - frameRight(c) / 2;
-            py = frameY(c);
+            px = frameX (c) + frameWidth (c) - frameRight(c) / 2;
+            py = frameY (c);
             break;
         case CORNER_COUNT + SIDE_LEFT:
-            px = frameX(c) + frameLeft(c) / 2;
-            py = frameY(c) + frameHeight(c) / 2;
+            px = frameX (c) + frameLeft(c) / 2;
+            py = frameY (c) + frameHeight (c) / 2;
             break;
         case CORNER_COUNT + SIDE_RIGHT:
-            px = frameX(c) + frameWidth(c) - frameRight(c) / 2;
-            py = frameY(c) + frameHeight(c) / 2;
+            px = frameX (c) + frameWidth (c) - frameRight(c) / 2;
+            py = frameY (c) + frameHeight (c) / 2;
             break;
         case CORNER_COUNT + SIDE_TOP:
-            px = frameX(c) + frameWidth(c) / 2;
-            py = frameY(c);
+            px = frameX (c) + frameWidth (c) / 2;
+            py = frameY (c);
             break;
         case CORNER_COUNT + SIDE_BOTTOM:
-            px = frameX(c) + frameWidth(c) / 2;
-            py = frameY(c) + frameHeight(c) - frameBottom(c) / 2;
+            px = frameX (c) + frameWidth (c) / 2;
+            py = frameY (c) + frameHeight (c) - frameBottom(c) / 2;
             break;
         default:
-            px = frameX(c) + frameWidth(c) / 2;
-            py = frameY(c) + frameHeight(c) / 2;
+            px = frameX (c) + frameWidth (c) / 2;
+            py = frameY (c) + frameHeight (c) / 2;
             break;
     }
 
@@ -221,6 +222,8 @@ clientSetHandle(MoveResizeData *passdata, int handle)
     passdata->handle = handle;
     passdata->mx = px;
     passdata->my = py;
+    passdata->px = passdata->mx - frameX (c);
+    passdata->py = passdata->my - frameY (c);
     passdata->ox = c->x;
     passdata->oy = c->y;
     passdata->ow = c->width;
@@ -753,10 +756,18 @@ clientMoveTile (Client *c, XMotionEvent *xevent)
     ScreenInfo *screen_info;
     GdkRectangle rect;
     int x, y, disp_x, disp_y, disp_max_x, disp_max_y, dist;
+    NetWmDesktopLayout layout;
 
     screen_info = c->screen_info;
 
-    if (screen_info->params->wrap_windows)
+    /*
+     * Tiling when moving really relies on restore_on_move to work
+     * reliably so just don't do anything if any of the above requirement
+     * is not met (restore_on_move being disabled is from another time,
+     * we should really not have such an option, I must have been weaked
+     * in the past...)
+     */
+    if (!(screen_info->params->tile_on_move && screen_info->params->restore_on_move))
     {
         return FALSE;
     }
@@ -770,37 +781,39 @@ clientMoveTile (Client *c, XMotionEvent *xevent)
     disp_max_x = rect.x + rect.width;
     disp_max_y = rect.y + rect.height;
 
+    layout = screen_info->desktop_layout;
     dist = MIN (TILE_DISTANCE, frameDecorationTop (screen_info) / 2);
-    if ((x >= disp_x - 1) && (x < disp_x + dist) &&
-        (y >= disp_y - 1) && (y < disp_max_y + 1))
+
+    if (!screen_info->params->wrap_windows || layout.cols < 2)
     {
-        TRACE ("event (%i,%i) monitor (%i,%i) %ix%i tile LEFT", x, y, disp_x, disp_y, disp_max_x, disp_max_y);
-        clientTile (c, x, y, TILE_LEFT, !screen_info->params->box_move);
-        return TRUE;
+        if ((x >= disp_x - 1) && (x < disp_x + dist) &&
+            (y >= disp_y - 1) && (y < disp_max_y + 1))
+        {
+            return clientTile (c, x, y, TILE_LEFT, !screen_info->params->box_move);
+        }
+
+        if ((x >= disp_max_x - dist) && (x < disp_max_x + 1) &&
+            (y >= disp_y - 1) && (y < disp_max_y + 1))
+        {
+            return clientTile (c, x, y, TILE_RIGHT, !screen_info->params->box_move);
+        }
     }
-    if ((x >= disp_max_x - dist) && (x < disp_max_x + 1) &&
-        (y >= disp_y - 1) && (y < disp_max_y + 1))
+
+    if (!screen_info->params->wrap_windows || layout.rows < 2)
     {
-        TRACE ("event (%i,%i) monitor (%i,%i) %ix%i tile RIGHT", x, y, disp_x, disp_y, disp_max_x, disp_max_y);
-        clientTile (c, x, y, TILE_RIGHT, !screen_info->params->box_move);
-        return TRUE;
-    }
-    if ((x >= disp_x - 1) && (x < disp_max_x + 1) &&
-        (y >= disp_y - 1) && (y < disp_y + dist))
-    {
-        TRACE ("event (%i,%i) monitor (%i,%i) %ix%i tile UP", x, y, disp_x, disp_y, disp_max_x, disp_max_y);
-        clientTile (c, x, y, TILE_UP, !screen_info->params->box_move);
-        return TRUE;
-    }
-    if ((x >= disp_x - 1) && (x < disp_max_x + 1) &&
-        (y >= disp_max_y - dist) && (y < disp_max_y + 1))
-    {
-        TRACE ("event (%i,%i) monitor (%i,%i) %ix%i tile DOWN", x, y, disp_x, disp_y, disp_max_x, disp_max_y);
-        clientTile (c, x, y, TILE_DOWN, !screen_info->params->box_move);
-        return TRUE;
+        if ((x >= disp_x - 1) && (x < disp_max_x + 1) &&
+            (y >= disp_y - 1) && (y < disp_y + dist))
+        {
+            return clientTile (c, x, y, TILE_UP, !screen_info->params->box_move);
+        }
+
+        if ((x >= disp_x - 1) && (x < disp_max_x + 1) &&
+            (y >= disp_max_y - dist) && (y < disp_max_y + 1))
+        {
+            return clientTile (c, x, y, TILE_DOWN, !screen_info->params->box_move);
+        }
     }
 
-    TRACE ("event (%i,%i) monitor (%i,%i) %ix%i *no* tile", x, y, disp_x, disp_y, disp_max_x, disp_max_y);
     return FALSE;
 }
 
@@ -814,7 +827,7 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
     eventFilterStatus status = EVENT_FILTER_STOP;
     MoveResizeData *passdata = (MoveResizeData *) data;
     Client *c = NULL;
-    gboolean moving = TRUE;
+    gboolean moving;
     XWindowChanges wc;
     int prev_x, prev_y;
 
@@ -827,6 +840,12 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
     display_info = screen_info->display_info;
     configure_flags = NO_CFG_FLAG;
 
+    /*
+     * Clients may choose to end the move operation,
+     * we use XFWM_FLAG_MOVING_RESIZING for that.
+     */
+    moving = FLAG_TEST (c->xfwm_flags, XFWM_FLAG_MOVING_RESIZING);
+
     /* Update the display time */
     myDisplayUpdateCurrentTime (display_info, xevent);
 
@@ -891,9 +910,11 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
             if (toggled_maximize)
             {
                 toggled_maximize = FALSE;
-                clientToggleMaximized (c, CLIENT_FLAG_MAXIMIZED, FALSE);
-                configure_flags = CFG_FORCE_REDRAW;
-                passdata->move_resized = TRUE;
+                if (clientToggleMaximized (c, CLIENT_FLAG_MAXIMIZED, FALSE))
+                {
+                    configure_flags = CFG_FORCE_REDRAW;
+                    passdata->move_resized = TRUE;
+                }
             }
         }
         else if (passdata->use_keys)
@@ -935,23 +956,35 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
                 (ABS (xevent->xmotion.y_root - passdata->my) > 15))
             {
                 /* to keep the distance from the edges of the window proportional. */
-                double xratio = .5;
+                double xratio, yratio;
 
-                if (FLAG_TEST(c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ))
+                xratio = (xevent->xmotion.x_root - frameX (c)) / (double) frameWidth (c);
+                yratio = (xevent->xmotion.y_root - frameY (c)) / (double) frameHeight (c);
+
+                if (FLAG_TEST_ALL(c->flags, CLIENT_FLAG_MAXIMIZED))
                 {
-                    xratio = (xevent->xmotion.x_root - c->x) / (double)c->width;
-                    if (FLAG_TEST(c->flags, CLIENT_FLAG_MAXIMIZED_VERT))
+                    toggled_maximize = TRUE;
+                }
+                if (clientToggleMaximized (c, c->flags & CLIENT_FLAG_MAXIMIZED, FALSE))
+                {
+                    passdata->move_resized = TRUE;
+
+                    passdata->ox = c->x;
+                    passdata->mx =  frameX (c) + passdata->px;
+                    if ((passdata->mx <  frameX (c)) || (passdata->mx >  frameX (c) + frameWidth (c)))
                     {
-                       toggled_maximize = TRUE;
+                        passdata->mx = CLAMP(frameX (c) + frameWidth (c) * xratio, frameX (c), frameX (c) + frameWidth (c));
                     }
+
+                    passdata->oy = c->y;
+                    passdata->my = frameY (c) + passdata->py;
+                    if ((passdata->my < frameY (c)) || (passdata->my > frameY (c) + frameHeight (c)))
+                    {
+                        passdata->my = CLAMP(frameY (c) + frameHeight (c) * yratio, frameY (c), frameY (c) + frameHeight (c));
+                    }
+
+                    configure_flags = CFG_FORCE_REDRAW;
                 }
-                clientToggleMaximized (c, c->flags & CLIENT_FLAG_MAXIMIZED, FALSE);
-                passdata->move_resized = TRUE;
-                passdata->ox = c->x;
-                passdata->mx = CLAMP(c->x + c->width * xratio, c->x, c->x + c->width);
-                passdata->oy = c->y;
-                passdata->my = c->y - frameTop(c) / 2;
-                configure_flags = CFG_FORCE_REDRAW;
             }
             else
             {
@@ -964,19 +997,14 @@ clientMoveEventFilter (XEvent * xevent, gpointer data)
         c->y = passdata->oy + (xevent->xmotion.y_root - passdata->my);
 
         clientSnapPosition (c, prev_x, prev_y);
-        if ((screen_info->params->restore_on_move) && toggled_maximize)
+        if (screen_info->params->restore_on_move && toggled_maximize)
         {
-            if (clientConstrainPos (c, FALSE) & CLIENT_CONSTRAINED_TOP)
+            if ((clientConstrainPos (c, FALSE) & CLIENT_CONSTRAINED_TOP) &&
+                 clientToggleMaximized (c, CLIENT_FLAG_MAXIMIZED, FALSE))
             {
-                clientToggleMaximized (c, CLIENT_FLAG_MAXIMIZED, FALSE);
                 configure_flags = CFG_FORCE_REDRAW;
                 toggled_maximize = FALSE;
                 passdata->move_resized = TRUE;
-                /*
-                   Update "passdata->my" to the current value to
-                   allow "restore on move" to keep working next time
-                 */
-                passdata->my = c->y - frameTop(c) / 2;
             }
         }
         else if (!clientMoveTile (c, (XMotionEvent *) xevent))
@@ -1078,11 +1106,15 @@ clientMove (Client * c, XEvent * ev)
     passdata.is_transient = clientIsValidTransientOrModal (c);
     passdata.move_resized = FALSE;
 
+    clientSaveSizePos (c);
+
     if (ev && (ev->type == ButtonPress))
     {
         passdata.button = ev->xbutton.button;
         passdata.mx = ev->xbutton.x_root;
         passdata.my = ev->xbutton.y_root;
+        passdata.px = passdata.mx - frameX (c);
+        passdata.py = passdata.my - frameY (c);
     }
     else
     {
@@ -1126,6 +1158,9 @@ clientMove (Client * c, XEvent * ev)
      */
     placeSidewalks(screen_info, FALSE);
 
+    /* Clear any previously saved pos flag from screen resize */
+    FLAG_UNSET (c->xfwm_flags, XFWM_FLAG_SAVED_POS);
+
     FLAG_SET (c->xfwm_flags, XFWM_FLAG_MOVING_RESIZING);
     TRACE ("entering move loop");
     eventFilterPush (display_info->xfilter, clientMoveEventFilter, &passdata);
@@ -1135,7 +1170,7 @@ clientMove (Client * c, XEvent * ev)
     FLAG_UNSET (c->xfwm_flags, XFWM_FLAG_MOVING_RESIZING);
 
     /* Put back the sidewalks as they ought to be */
-    placeSidewalks(screen_info, screen_info->params->wrap_workspaces);
+    placeSidewalks (screen_info, screen_info->params->wrap_workspaces);
 
 #ifdef SHOW_POSITION
     if (passdata.poswin)
@@ -1265,7 +1300,12 @@ clientResizeEventFilter (XEvent * xevent, gpointer data)
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
     status = EVENT_FILTER_STOP;
-    resizing = TRUE;
+
+    /*
+     * Clients may choose to end the resize operation,
+     * we use XFWM_FLAG_MOVING_RESIZING for that.
+     */
+    resizing = FLAG_TEST (c->xfwm_flags, XFWM_FLAG_MOVING_RESIZING);
 
     frame_x = frameX (c);
     frame_y = frameY (c);
@@ -1690,6 +1730,9 @@ clientResize (Client * c, int handle, XEvent * ev)
         clientSetOpacity (c, c->opacity, OPACITY_RESIZE, OPACITY_RESIZE);
     }
 
+    /* Clear any previously saved pos flag from screen resize */
+    FLAG_UNSET (c->xfwm_flags, XFWM_FLAG_SAVED_POS);
+
     FLAG_SET (c->xfwm_flags, XFWM_FLAG_MOVING_RESIZING);
     TRACE ("entering resize loop");
     eventFilterPush (display_info->xfilter, clientResizeEventFilter, &passdata);
