@@ -133,11 +133,6 @@ clientSetNetState (Client * c)
     XChangeProperty (display_info->dpy, c->window,
                      display_info->atoms[NET_WM_STATE], XA_ATOM, 32,
                      PropModeReplace, (unsigned char *) data, i);
-    /*
-       We also set GNOME hint here for consistency and convenience,
-       although the meaning of net_wm_state and win_state aren't the same.
-     */
-    setHint (display_info, c->window, WIN_STATE, c->win_state);
 }
 
 void
@@ -162,25 +157,21 @@ clientGetNetState (Client * c)
         if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
         {
             TRACE ("clientGetNetState : shaded from session management");
-            c->win_state |= WIN_STATE_SHADED;
             FLAG_SET (c->flags, CLIENT_FLAG_SHADED);
         }
         if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
         {
             TRACE ("clientGetNetState : sticky from session management");
-            c->win_state |= WIN_STATE_STICKY;
             FLAG_SET (c->flags, CLIENT_FLAG_STICKY);
         }
         if (FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ))
         {
             TRACE ("clientGetNetState : maximized horiz from session management");
-            c->win_state |= WIN_STATE_MAXIMIZED_HORIZ;
             FLAG_SET (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ);
         }
         if (FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_VERT))
         {
             TRACE ("clientGetNetState : maximized vert from session management");
-            c->win_state |= WIN_STATE_MAXIMIZED_VERT;
             FLAG_SET (c->flags, CLIENT_FLAG_MAXIMIZED_VERT);
         }
     }
@@ -196,25 +187,21 @@ clientGetNetState (Client * c)
             if ((atoms[i] == display_info->atoms[NET_WM_STATE_SHADED]))
             {
                 TRACE ("clientGetNetState : shaded");
-                c->win_state |= WIN_STATE_SHADED;
                 FLAG_SET (c->flags, CLIENT_FLAG_SHADED);
             }
             else if ((atoms[i] == display_info->atoms[NET_WM_STATE_STICKY]))
             {
                 TRACE ("clientGetNetState : sticky");
-                c->win_state |= WIN_STATE_STICKY;
                 FLAG_SET (c->flags, CLIENT_FLAG_STICKY);
             }
             else if ((atoms[i] == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]))
             {
                 TRACE ("clientGetNetState : maximized horiz");
-                c->win_state |= WIN_STATE_MAXIMIZED_HORIZ;
                 FLAG_SET (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ);
             }
             else if ((atoms[i] == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]))
             {
                 TRACE ("clientGetNetState : maximized vert");
-                c->win_state |= WIN_STATE_MAXIMIZED_VERT;
                 FLAG_SET (c->flags, CLIENT_FLAG_MAXIMIZED_VERT);
             }
             else if ((atoms[i] == display_info->atoms[NET_WM_STATE_FULLSCREEN]))
@@ -283,6 +270,43 @@ clientGetNetState (Client * c)
     }
 }
 
+void
+clientUpdateNetWmDesktop (Client * c, XClientMessageEvent * ev)
+{
+    ScreenInfo *screen_info;
+
+    g_return_if_fail (c != NULL);
+    TRACE ("entering clientUpdateNetWmDesktop");
+    TRACE ("client \"%s\" (0x%lx), value 0x%lx", c->name, c->window, ev->data.l[0]);
+
+    screen_info = c->screen_info;
+
+    if ((guint) ev->data.l[0] == ALL_WORKSPACES)
+    {
+        if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK) && !FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
+        {
+            clientStick (c, TRUE);
+            frameQueueDraw (c, FALSE);
+        }
+    }
+    else if ((guint) ev->data.l[0] < (guint) screen_info->workspace_count)
+    {
+        if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_STICK) && FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
+        {
+            clientUnstick (c, TRUE);
+            frameQueueDraw (c, FALSE);
+        }
+        if ((guint) ev->data.l[0] != (guint) c->win_workspace)
+        {
+            clientSetWorkspace (c, (guint) ev->data.l[0], TRUE);
+        }
+    }
+    else
+    {
+        TRACE ("Ignoring invalid NET_WM_DESKTOP value 0x%lx specified for client \"%s\" (0x%lx)", ev->data.l[0], c->name, c->window);
+    }
+}
+
 void
 clientUpdateNetState (Client * c, XClientMessageEvent * ev)
 {
@@ -351,16 +375,16 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
         {
             if ((action == NET_WM_STATE_ADD) && !FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED))
             {
-                mode = 0;
+                mode = 0L;
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]))
                 {
-                    mode |= !FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ) ? WIN_STATE_MAXIMIZED_HORIZ : 0;
+                    mode |= !FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ) ? CLIENT_FLAG_MAXIMIZED_HORIZ : 0;
                 }
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]))
                 {
-                    mode |= !FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_VERT) ? WIN_STATE_MAXIMIZED_VERT : 0;
+                    mode |= !FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_VERT) ? CLIENT_FLAG_MAXIMIZED_VERT : 0;
                 }
                 clientToggleMaximized (c, mode, TRUE);
             }
@@ -370,12 +394,12 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]))
                 {
-                    mode |= FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ) ? WIN_STATE_MAXIMIZED_HORIZ : 0;
+                    mode |= FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_HORIZ);
                 }
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]))
                 {
-                    mode |= FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_VERT) ? WIN_STATE_MAXIMIZED_VERT : 0;
+                    mode |= FLAG_TEST (c->flags, CLIENT_FLAG_MAXIMIZED_VERT);
                 }
                 clientToggleMaximized (c, mode, TRUE);
             }
@@ -385,12 +409,12 @@ clientUpdateNetState (Client * c, XClientMessageEvent * ev)
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_HORZ]))
                 {
-                    mode |= WIN_STATE_MAXIMIZED_HORIZ;
+                    mode |= CLIENT_FLAG_MAXIMIZED_HORIZ;
                 }
                 if ((first  == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]) ||
                     (second == display_info->atoms[NET_WM_STATE_MAXIMIZED_VERT]))
                 {
-                    mode |= WIN_STATE_MAXIMIZED_VERT;
+                    mode |= CLIENT_FLAG_MAXIMIZED_VERT;
                 }
                 clientToggleMaximized (c, mode, TRUE);
             }
@@ -671,10 +695,15 @@ clientNetMoveResize (Client * c, XClientMessageEvent * ev)
             resize = TRUE; /* Resize */
             break;
         case NET_WM_MOVERESIZE_MOVE:
-        default:
             event->type = ButtonPress;
             resize = FALSE; /* Move */
             break;
+        case NET_WM_MOVERESIZE_CANCEL:
+            FLAG_UNSET (c->xfwm_flags, XFWM_FLAG_MOVING_RESIZING);
+            /* Walk through */
+        default: /* Do nothing */
+            return;
+            break;
     }
 
     if (!FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
@@ -862,7 +891,6 @@ clientGetInitialNetWmDesktop (Client * c)
             {
                 TRACE ("atom net_wm_desktop specifies window \"%s\" is sticky", c->name);
                 FLAG_SET (c->flags, CLIENT_FLAG_STICKY);
-                c->win_state |= WIN_STATE_STICKY;
             }
             c->win_workspace = c->screen_info->current_ws;
         }
@@ -873,12 +901,6 @@ clientGetInitialNetWmDesktop (Client * c)
         }
         FLAG_SET (c->xfwm_flags, XFWM_FLAG_WORKSPACE_SET);
     }
-    else if (getHint (display_info, c->window, WIN_WORKSPACE, &val))
-    {
-        TRACE ("atom win_workspace specifies window \"%s\" is on desk %i", c->name, (int) val);
-        c->win_workspace = (int) val;
-        FLAG_SET (c->xfwm_flags, XFWM_FLAG_WORKSPACE_SET);
-    }
 
     /* This is to make sure that transient are shown with their "ancestor" window */
     if (!FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
@@ -891,7 +913,6 @@ clientGetInitialNetWmDesktop (Client * c)
             if (FLAG_TEST (c2->flags, CLIENT_FLAG_STICKY))
             {
                 FLAG_SET (c->flags, CLIENT_FLAG_STICKY);
-                c->win_state |= WIN_STATE_STICKY;
                 c->win_workspace = c->screen_info->current_ws;
             }
         }
@@ -905,7 +926,6 @@ clientGetInitialNetWmDesktop (Client * c)
         FLAG_SET (c->xfwm_flags, XFWM_FLAG_WORKSPACE_SET);
     }
     TRACE ("initial desktop for window \"%s\" is %i", c->name, c->win_workspace);
-    setHint (display_info, c->window, WIN_WORKSPACE, c->win_workspace);
     if (FLAG_TEST (c->flags, CLIENT_FLAG_STICKY))
     {
         setHint (display_info, c->window, NET_WM_DESKTOP, (unsigned long) ALL_WORKSPACES);
@@ -1193,7 +1213,6 @@ clientWindowType (Client * c)
             TRACE ("atom net_wm_window_type_desktop detected");
             c->type = WINDOW_DESKTOP;
             c->initial_layer = WIN_LAYER_DESKTOP;
-            c->win_state |= WIN_STATE_STICKY;
             FLAG_SET (c->flags,
                 CLIENT_FLAG_SKIP_PAGER | CLIENT_FLAG_STICKY |
                 CLIENT_FLAG_SKIP_TASKBAR);
