@@ -32,15 +32,46 @@
 #define WIN_ICON_BORDER 5
 #endif
 
+#ifndef WIN_COLOR_BORDER
+#define WIN_COLOR_BORDER 3
+#endif
+
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
 #include <libxfce4util/libxfce4util.h>
-#include "inline-tabwin-icon.h"
 #include "icons.h"
 #include "focus.h"
 #include "tabwin.h"
 
+static void tabwin_widget_class_init (TabwinWidgetClass *klass);
+
+static GType
+tabwin_widget_get_type (void)
+{
+    static GType type = G_TYPE_INVALID;
+
+    if (G_UNLIKELY (type == G_TYPE_INVALID))
+    {
+        static const GTypeInfo info =
+        {
+            sizeof (TabwinWidgetClass),
+            NULL,
+            NULL,
+            (GClassInitFunc) tabwin_widget_class_init,
+            NULL,
+            NULL,
+            sizeof (TabwinWidget),
+            0,
+            NULL,
+            NULL,
+        };
+
+        type = g_type_register_static (GTK_TYPE_WINDOW, "Xfwm4TabwinWidget", &info, 0);
+    }
+
+    return type;
+}
 
 static GdkColor *
 get_color (GtkWidget *win, GtkStateType state_type)
@@ -174,7 +205,7 @@ tabwinSetSelected (TabwinWidget *tbw, GtkWidget *w)
 }
 
 static GtkWidget *
-createWindowIcon (Client *c)
+createWindowIcon (Client *c, gint icon_size)
 {
     GdkPixbuf *icon_pixbuf;
     GdkPixbuf *icon_pixbuf_stated;
@@ -183,7 +214,7 @@ createWindowIcon (Client *c)
     g_return_val_if_fail (c, NULL);
     TRACE ("entering createWindowIcon");
 
-    icon_pixbuf = getAppIcon (c->screen_info->display_info, c->window, WIN_ICON_SIZE, WIN_ICON_SIZE);
+    icon_pixbuf = getAppIcon (c->screen_info->display_info, c->window, icon_size, icon_size);
     icon_pixbuf_stated = NULL;
     icon = gtk_image_new ();
     g_object_set_data (G_OBJECT (icon), "client-ptr-val", c);
@@ -211,15 +242,29 @@ createWindowIcon (Client *c)
     return icon;
 }
 
+static int
+getMinMonitorWidth (ScreenInfo *screen_info)
+{
+    int i, min_width, num_monitors = myScreenGetNumMonitors (screen_info);
+    for (min_width = i = 0; i < num_monitors; i++)
+    {
+        GdkRectangle monitor;
+        gdk_screen_get_monitor_geometry (screen_info->gscr, i, &monitor);
+        if (min_width == 0 || monitor.width < min_width)
+            min_width = monitor.width;
+    }
+    return min_width;
+}
+
 static GtkWidget *
 createWindowlist (ScreenInfo *screen_info, TabwinWidget *tbw)
 {
     Client *c;
     GList *client_list;
-    GdkRectangle monitor;
     GtkWidget *windowlist, *icon, *selected;
-    int packpos;
+    int packpos, monitor_width;
     Tabwin *t;
+    gint icon_size = WIN_ICON_SIZE;
 
     TRACE ("entering createWindowlist");
 
@@ -227,11 +272,11 @@ createWindowlist (ScreenInfo *screen_info, TabwinWidget *tbw)
     c = NULL;
     selected = NULL;
     t = tbw->tabwin;
+    monitor_width = getMinMonitorWidth (screen_info);
     g_return_val_if_fail (screen_info->client_count > 0, NULL);
 
-    gdk_screen_get_monitor_geometry (screen_info->gscr, tbw->monitor_num, &monitor);
-
-    tbw->grid_cols = (monitor.width / (WIN_ICON_SIZE + 2 * WIN_ICON_BORDER)) * 0.75;
+    gtk_widget_style_get (GTK_WIDGET (tbw), "icon-size", &icon_size, NULL);
+    tbw->grid_cols = (monitor_width / (icon_size + 2 * WIN_ICON_BORDER)) * 0.75;
     tbw->grid_rows = screen_info->client_count / tbw->grid_cols + 1;
     tbw->widgets = NULL;
     windowlist = gtk_table_new (tbw->grid_rows, tbw->grid_cols, FALSE);
@@ -241,7 +286,7 @@ createWindowlist (ScreenInfo *screen_info, TabwinWidget *tbw)
     {
         c = (Client *) client_list->data;
         TRACE ("createWindowlist: adding %s", c->name);
-        icon = createWindowIcon (c);
+        icon = createWindowIcon (c, icon_size);
         gtk_table_attach (GTK_TABLE (windowlist), GTK_WIDGET (icon),
             packpos % tbw->grid_cols, packpos % tbw->grid_cols + 1,
             packpos / tbw->grid_cols, packpos / tbw->grid_cols + 1,
@@ -277,7 +322,7 @@ tabwinConfigure (TabwinWidget *tbw, GdkEventConfigure *event)
         return FALSE;
     }
 
-    window = GTK_WINDOW(tbw->window);
+    window = GTK_WINDOW(tbw);
     screen = gtk_window_get_screen(window);
     gdk_screen_get_monitor_geometry (screen, tbw->monitor_num, &monitor);
     x = monitor.x + (monitor.width - event->width) / 2;
@@ -290,6 +335,27 @@ tabwinConfigure (TabwinWidget *tbw, GdkEventConfigure *event)
     return FALSE;
 }
 
+static void
+tabwin_widget_class_init (TabwinWidgetClass *klass)
+{
+    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+
+    gtk_widget_class_install_style_property (widget_class,
+                                             g_param_spec_int("icon-size",
+                                                              "icon size",
+                                                               "the size of the application icon",
+                                                               24, 128,
+                                                               WIN_ICON_SIZE,
+                                                               G_PARAM_READABLE));
+    gtk_widget_class_install_style_property (widget_class,
+                                             g_param_spec_int("border-width",
+                                                              "border width",
+                                                               "the width of the colored border",
+                                                               0, 8,
+                                                               WIN_COLOR_BORDER,
+                                                               G_PARAM_READABLE));
+}
+
 static TabwinWidget *
 tabwinCreateWidget (Tabwin *tabwin, ScreenInfo *screen_info, gint monitor_num)
 {
@@ -300,12 +366,12 @@ tabwinCreateWidget (Tabwin *tabwin, ScreenInfo *screen_info, gint monitor_num)
     GtkWidget *windowlist;
     GdkColor *color;
     GdkRectangle monitor;
+    gint border_width = WIN_COLOR_BORDER;
 
     TRACE ("entering tabwinCreateWidget for monitor %i", monitor_num);
 
-    tbw = g_new0 (TabwinWidget, 1);
+    tbw = g_object_new (tabwin_widget_get_type(), "type", GTK_WINDOW_POPUP, NULL);
 
-    tbw->window = gtk_window_new (GTK_WINDOW_POPUP);
     tbw->width = -1;
     tbw->height = -1;
     tbw->monitor_num = monitor_num;
@@ -313,25 +379,26 @@ tabwinCreateWidget (Tabwin *tabwin, ScreenInfo *screen_info, gint monitor_num)
     tbw->selected = NULL;
     tbw->selected_callback = 0;
 
-    gtk_window_set_screen (GTK_WINDOW (tbw->window), screen_info->gscr);
-    gtk_widget_set_name (GTK_WIDGET (tbw->window), "xfwm4-tabwin");
-    gtk_widget_realize (GTK_WIDGET (tbw->window));
-    gtk_container_set_border_width (GTK_CONTAINER (tbw->window), 0);
-    gtk_window_set_position (GTK_WINDOW (tbw->window), GTK_WIN_POS_NONE);
+    gtk_window_set_screen (GTK_WINDOW (tbw), screen_info->gscr);
+    gtk_widget_set_name (GTK_WIDGET (tbw), "xfwm4-tabwin");
+    gtk_widget_realize (GTK_WIDGET (tbw));
+    gtk_container_set_border_width (GTK_CONTAINER (tbw), 0);
+    gtk_window_set_position (GTK_WINDOW (tbw), GTK_WIN_POS_NONE);
     gdk_screen_get_monitor_geometry (screen_info->gscr, tbw->monitor_num, &monitor);
-    gtk_window_move (GTK_WINDOW(tbw->window), monitor.x + monitor.width / 2,
-                                              monitor.y + monitor.height / 2);
+    gtk_window_move (GTK_WINDOW(tbw), monitor.x + monitor.width / 2,
+                                      monitor.y + monitor.height / 2);
 
     frame = gtk_frame_new (NULL);
     gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
     gtk_container_set_border_width (GTK_CONTAINER (frame), 0);
-    gtk_container_add (GTK_CONTAINER (tbw->window), frame);
+    gtk_container_add (GTK_CONTAINER (tbw), frame);
 
     colorbox1 = gtk_event_box_new ();
     gtk_container_add (GTK_CONTAINER (frame), colorbox1);
 
+    gtk_widget_style_get (GTK_WIDGET (tbw), "border-width", &border_width, NULL);
     colorbox2 = gtk_event_box_new ();
-    gtk_container_set_border_width (GTK_CONTAINER (colorbox2), 3);
+    gtk_container_set_border_width (GTK_CONTAINER (colorbox2), border_width);
     gtk_container_add (GTK_CONTAINER (colorbox1), colorbox2);
 
     vbox = gtk_vbox_new (FALSE, 5);
@@ -359,7 +426,7 @@ tabwinCreateWidget (Tabwin *tabwin, ScreenInfo *screen_info, gint monitor_num)
     tbw->container = windowlist;
     gtk_container_add (GTK_CONTAINER (frame), windowlist);
 
-    color = get_color (tbw->window, GTK_STATE_SELECTED);
+    color = get_color (GTK_WIDGET (tbw), GTK_STATE_SELECTED);
     if (color)
     {
         gtk_widget_modify_bg (colorbox1, GTK_STATE_NORMAL, color);
@@ -373,14 +440,38 @@ tabwinCreateWidget (Tabwin *tabwin, ScreenInfo *screen_info, gint monitor_num)
     }
 #endif
 
-    g_signal_connect_swapped (tbw->window, "configure-event",
+    g_signal_connect_swapped (tbw, "configure-event",
                               GTK_SIGNAL_FUNC (tabwinConfigure), (gpointer) tbw);
 
-    gtk_widget_show_all (GTK_WIDGET (tbw->window));
+    gtk_widget_show_all (GTK_WIDGET (tbw));
 
     return tbw;
 }
 
+static Client *
+tabwinChange2Selected (Tabwin *t, GList *selected)
+{
+    GList *tabwin_list, *widgets;
+    GtkWidget *icon;
+    TabwinWidget *tbw;
+
+    t->selected = selected;
+    for (tabwin_list = t->tabwin_list; tabwin_list; tabwin_list = g_list_next (tabwin_list))
+    {
+        tbw = (TabwinWidget *) tabwin_list->data;
+        for (widgets = tbw->widgets; widgets; widgets = g_list_next (widgets))
+        {
+            icon = GTK_WIDGET (widgets->data);
+            if (((Client *) g_object_get_data (G_OBJECT(icon), "client-ptr-val")) == t->selected->data)
+            {
+                    tabwinSetSelected (tbw, icon);
+                    gtk_widget_queue_draw (GTK_WIDGET(tbw));
+            }
+        }
+    }
+    return tabwinGetSelected (t);
+}
+
 Tabwin *
 tabwinCreate (GList **client_list, GList *selected, gboolean display_workspace)
 {
@@ -501,7 +592,7 @@ tabwinSelectHead (Tabwin *t)
             if (((Client *) g_object_get_data (G_OBJECT(icon), "client-ptr-val")) == head->data)
                 {
                     tabwinSetSelected (tbw, icon);
-                    gtk_widget_queue_draw (tbw->window);
+                    gtk_widget_queue_draw (GTK_WIDGET(tbw));
                 }
             }
     }
@@ -513,9 +604,6 @@ Client *
 tabwinSelectNext (Tabwin *t)
 {
     GList *next;
-    GList *tabwin_list, *widgets;
-    GtkWidget *icon;
-    TabwinWidget *tbw;
 
     g_return_val_if_fail (t != NULL, NULL);
     TRACE ("entering tabwinSelectNext");
@@ -526,31 +614,13 @@ tabwinSelectNext (Tabwin *t)
         next = *t->client_list;
         g_return_val_if_fail (next != NULL, NULL);
     }
-    t->selected = next;
-    for (tabwin_list = t->tabwin_list; tabwin_list; tabwin_list = g_list_next (tabwin_list))
-    {
-        tbw = (TabwinWidget *) tabwin_list->data;
-        for (widgets = tbw->widgets; widgets; widgets = g_list_next (widgets))
-        {
-            icon = GTK_WIDGET (widgets->data);
-            if (((Client *) g_object_get_data (G_OBJECT(icon), "client-ptr-val")) == next->data)
-                {
-                    tabwinSetSelected (tbw, icon);
-                    gtk_widget_queue_draw (tbw->window);
-                }
-            }
-    }
-
-    return tabwinGetSelected (t);
+    return tabwinChange2Selected (t, next);
 }
 
 Client *
 tabwinSelectPrev (Tabwin *t)
 {
     GList *prev;
-    GList *tabwin_list, *widgets;
-    GtkWidget *icon;
-    TabwinWidget *tbw;
 
     g_return_val_if_fail (t != NULL, NULL);
     TRACE ("entering tabwinSelectPrev");
@@ -561,22 +631,83 @@ tabwinSelectPrev (Tabwin *t)
         prev = g_list_last (*t->client_list);
         g_return_val_if_fail (prev != NULL, NULL);
     }
-    t->selected = prev;
-    for (tabwin_list = t->tabwin_list; tabwin_list; tabwin_list = g_list_next (tabwin_list))
+    return tabwinChange2Selected (t, prev);
+}
+
+Client *
+tabwinSelectDelta (Tabwin *t, int row_delta, int col_delta)
+{
+    GList *selected;
+    int pos_current, col_current, row_current, nitems, cols, rows;
+    TabwinWidget *tbw;
+
+    TRACE ("entering tabwinSelectDelta");
+    g_return_val_if_fail (t != NULL, NULL);
+
+    tbw = (TabwinWidget *) t->tabwin_list->data;
+    pos_current = g_list_position (*t->client_list, t->selected);
+    nitems = g_list_length (*t->client_list);
+
+    cols = MIN (nitems, tbw->grid_cols);
+    rows = (nitems - 1) / cols + 1;
+    col_current = pos_current % cols;
+    row_current = pos_current / cols;
+
+    /* Wrap column */
+    col_current += col_delta;
+    if (col_current < 0)
     {
-        tbw = (TabwinWidget *) tabwin_list->data;
-        for (widgets = tbw->widgets; widgets; widgets = g_list_next (widgets))
-        {
-            icon = GTK_WIDGET (widgets->data);
-            if (((Client *) g_object_get_data (G_OBJECT(icon), "client-ptr-val")) == prev->data)
-                {
-                    tabwinSetSelected (tbw, icon);
-                    gtk_widget_queue_draw (tbw->window);
-                }
-            }
+        col_current = cols - 1;
+    }
+    else if (col_current >= cols)
+    {
+        col_current = 0;
     }
 
-    return tabwinGetSelected (t);
+    /* Wrap row */
+    row_current += row_delta;
+    if (row_current < 0)
+    {
+        row_current = rows - 1;
+    }
+    else if (row_current >= rows)
+    {
+        row_current = 0;
+    }
+
+    /* So here we are at the new (wrapped) position in the rectangle */
+    pos_current = col_current + row_current * cols;
+    if (pos_current >= nitems)   /* If that position does not exist */
+    {
+        if (col_delta)            /* Let horizontal prevail */
+        {
+            if (col_delta > 0)
+            {
+                col_current = 0;
+            }
+            else
+            {
+                col_current = (nitems - 1) % cols;
+            }
+        }
+        else
+        {
+            if (row_delta > 0)
+            {
+                row_current = 0;
+            }
+            else
+            {
+                row_current = row_current - 1;
+            }
+
+        }
+        pos_current = col_current + row_current * cols;
+    }
+
+    selected = g_list_nth(*t->client_list, pos_current);
+
+    return tabwinChange2Selected (t, selected);
 }
 
 void
@@ -593,8 +724,8 @@ tabwinDestroy (Tabwin *t)
     {
         tbw = (TabwinWidget *) tabwin_list->data;
         g_list_free (tbw->widgets);
-        gtk_widget_destroy (tbw->window);
-        g_free (tbw);
+        gtk_widget_destroy (GTK_WIDGET (tbw));
     }
     g_list_free (t->tabwin_list);
 }
+
